import Flutter
import UIKit

public class SwiftSpectusInAppPurchasePlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "spectus_in_app_purchase", binaryMessenger: registrar.messenger())
        let instance = SwiftSpectusInAppPurchasePlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if (call.method == "getPlatformVersion") {
            result("iOS " + UIDevice.current.systemVersion)
        }
        else if (call.method == "startInAppPurchaseWith") {
            //In-App Purchase Payment
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any], let prodId = myArgs["productID"] as? String, let iosAppSecretVal = myArgs["iosAppSecret"] as? String {
                IAPHelper.shared.startInAppPurchaseWith(productIdentifier : prodId, iosAppSecret: iosAppSecretVal, { (errorMessage, transactionObj, transactionID) in
                    if errorMessage != nil {
                        result("NA")
                    } else {
                        result(transactionID)
                    }
                })
            }
        }
        else if (call.method == "startRestorePurchase") {
            //In-App Restore Purchase
            IAPHelper.shared.restorePurchase({ (errorMessage, transactionObj) in
                if errorMessage != nil {
                    result("NA")
                } else {
                    if let transactionID = transactionObj?.transactionIdentifier {
                        result(transactionID)
                    } else {
                        result("NA")
                    }
                }
            })
        }
        else if (call.method == "getProductDetailsOf") {
                    guard let args = call.arguments else {
                        return
                    }
                    if let myArgs = args as? [String: Any], let prodId = myArgs["productID"] as? String, let iosAppSecretVal = myArgs["iosAppSecret"] as? String {
                        IAPHelper.shared.getProductInfo(productIdentifier: prodId, iosAppSecret: iosAppSecretVal) { productIdentifier, productObject in
                            print("Completion In Swift => \n Product ID => \(productIdentifier)\n Product => \(productObject?.description ?? "")\n=======================\n")

                            if let product = productObject {
                                print("\n=> \(product.productIdentifier)")
                                print("\n=> \(product.price)")
                                print("\n=> \(product.priceLocale)")
                                print("\n=> \(product.priceLocale.currencyCode)")
                                print("\n=> \(product.priceLocale.currencySymbol)")
                                print("\n=> \(self.getSymbolForCurrencyCode(code: product.priceLocale.identifier))")
                                print("\n=> \(product.localizedTitle)")
                                print("\n=> \(product.localizedDescription)")

                                let formatter = NumberFormatter()
                                formatter.numberStyle = .currency
                                let priceWithFormat = formatter.string(from: product.price) ?? product.price.description

                                let productDetails : [String : String] = ["product_identifier": product.productIdentifier,
                                                      "product_price": priceWithFormat,
                                                      "currency_code": product.priceLocale.currencyCode ?? (Locale.current.currencyCode ?? ""),
                                                      "currency_symbol": product.priceLocale.currencySymbol ?? (Locale.current.currencySymbol ?? "")]
                                result(productDetails.description)
                            } else {
                                result("NA")
                            }
                        }
                    }
        }
        else {
            result("Failed to get top controller")
        }
    }

    func getSymbolForCurrencyCode(code: String) -> String {
        var candidates: [String] = []
        let locales: [String] = NSLocale.availableLocaleIdentifiers
        for localeID in locales {
            guard let symbol = findMatchingSymbol(localeID: localeID, currencyCode: code) else {
                continue
            }
            if symbol.count == 1 {
                return symbol
            }
            candidates.append(symbol)
        }
        let sorted = sortAscByLength(list: candidates)
        if sorted.count < 1 {
            return ""
        }
        return sorted[0]
    }

    func findMatchingSymbol(localeID: String, currencyCode: String) -> String? {
        let locale = Locale(identifier: localeID as String)
        guard let code = locale.currencyCode else {
            return nil
        }
        if code != currencyCode {
            return nil
        }
        guard let symbol = locale.currencySymbol else {
            return nil
        }
        return symbol
    }

    func sortAscByLength(list: [String]) -> [String] {
        return list.sorted(by: { $0.count < $1.count })
    }
}
